package ocean.gateway.routes;

import static com.github.tomakehurst.wiremock.client.WireMock.get;

import com.github.tomakehurst.wiremock.junit.WireMockRule;

import org.junit.After;
import org.junit.Before;
import org.junit.BeforeClass;
import org.junit.Rule;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.boot.test.web.client.TestRestTemplate;
import org.springframework.boot.web.server.LocalServerPort;
import org.springframework.http.HttpEntity;
import org.springframework.http.HttpHeaders;
import org.springframework.http.HttpMethod;
import org.springframework.http.HttpStatus;
import org.springframework.http.MediaType;
import org.springframework.http.ResponseEntity;
import org.springframework.test.context.junit4.SpringRunner;
import org.springframework.util.Assert;
import org.springframework.web.reactive.function.client.ClientResponse;
import org.springframework.web.reactive.function.client.WebClient;
import org.springframework.web.reactive.function.client.WebClient.RequestBodySpec;

import ocean.gateway.service.ServiceApplication;
import ocean.gateway.service.filters.TokenFilter;
import reactor.core.publisher.Mono;

import static com.github.tomakehurst.wiremock.client.WireMock.any;
import static com.github.tomakehurst.wiremock.client.WireMock.equalTo;
import static com.github.tomakehurst.wiremock.client.WireMock.get;
import static com.github.tomakehurst.wiremock.client.WireMock.*;

@RunWith(SpringRunner.class)
@SpringBootTest(classes = { ServiceApplication.class }, webEnvironment = SpringBootTest.WebEnvironment.RANDOM_PORT)
public class BusinessTests {

	// @RestController
	// @RequestMapping(value = "/my")
	// public class MyController {
	// @RequestMapping(value = "/integration/{name}", method = RequestMethod.GET)
	// public String integrationTest(@PathVariable String name) {
	// return "name:" + name;
	// }
	// }

	@LocalServerPort
	public int randomServerPort;
	public String baseUri;

	@Rule
	public WireMockRule routeTestServer = new WireMockRule(3030);

	@Before
	public void before() {

		baseUri = "http://127.0.0.1:" + randomServerPort;
		routeTestServer.stubFor(get(urlPathEqualTo("/hello")).willReturn(
				aResponse().withStatus(200).withBody("hello route")));
	}

	@After
	public void after() {
		routeTestServer.shutdown();
	}

	@Test
	public void TestRouteLocator() {


		WebClient cli = WebClient.create();
		ClientResponse response = cli.get().uri(baseUri + "/hello").header("token", TokenFilter.TEST_TOKEN).exchange().block();
		String content = response.bodyToMono(String.class).block();

		Assert.hasText(content, "hello route");
		Assert.notNull(response, "The class must not be null");
		Assert.state(response.statusCode() != HttpStatus.UNAUTHORIZED, "response.getStatusCode() must not be HttpStatus.UNAUTHORIZED");
		Assert.state(response.statusCode() == HttpStatus.OK, String.format("the statusCode is %s, not is %s\n", response.statusCode(), HttpStatus.OK));
	}

	

	// @Configuration
	// public class HttpServerConfig {
	// @Autowired
	// private Environment environment;
	// private String serverPort;
	// @Bean
	// public HttpServer httpServer() {
	// 	HttpServer server = HttpServer.create().host("http://localhost:" + serverPort);
	// 	server.handle((req, res) -> res.sendString(Flux.just("HelloTest")));
	// 	return server;
	// 	}
	// }





}

